from __future__ import print_function
import numpy as np
import os
import pandas as pd
import matplotlib.pyplot as plt
#from PS1QueryFunctions import *
import json
from astropy.coordinates import Angle
from astropy import units as u
import astropy.coordinates as coord

## between two dataframes, remove all the rows in the second one
def removeDuplicateSN_hostTable(host_DF, severalHosts):
    for host in severalHosts:
        host_DF_sub = host_DF[host_DF['NED_name'] == host]
        tempCoord = coord.SkyCoord(row.TransientRA*u.deg,row.TransientDEC*u.deg, frame='icrs')
        sep = coord.SkyCoord(host_DF_sub['TransientRA']*u.deg,host_DF_sub['TransientDEC']*u.deg, frame='icrs').separation(tempCoord)
        idx = np.argmin(sep)
        idx_DF = host_DF.index[idx]
        if (sep[idx].arcsec <= 1) and (row.TransientDiscoveryYear == host_DF_sub.loc[idx_DF, 'TransientDiscoveryYear']):
            #print('supernova to match is: ')
            #print(index)
            print("Matched:")
            print("%s" % row['TransientName'])
            print("%s" % row.TransientClass)
            print("%s "%host_DF_sub.loc[idx_DF, 'TransientName'])
            print("%s" %host_DF_sub.loc[idx_DF, 'TransientClass'])
            host_DF.drop([idx_DF], inplace=True)
            host_DF.reset_index(inplace=True, drop=True)
        else:
            print("No matches, smallest separation is:")
            print(sep[idx].arcsec)
    return host_DF

## between two dataframes, remove all the rows in the second one
def find_unique_transients(transients, OSC_df):
    for index, row in transients.iterrows():
        print(index)
        tempCoord = coord.SkyCoord(row.RA_deg*u.deg,row.DEC_deg*u.deg, frame='icrs')
        sep = coord.SkyCoord(OSC_df['RA_deg']*u.deg,OSC_df['DEC_deg']*u.deg, frame='icrs').separation(tempCoord)
        idx = np.argmin(sep)

        if sep[idx].arcsec < 2 and row.Year == OSC_df.loc[idx, 'Year']:
            #print('supernova to match is: ')
            #print(index)
            print("TNS SNe: %s" % row.Name)
            print("TNS Type: %s" % row.ObjType)
            #print("from")
            #print(row.Source)
            #print("supernova matched is:")
            #print(idx)
            print("OSC SNe: %s "%OSC_df.loc[idx, 'Name'])
            print("OSC Type: %s" %OSC_df.loc[idx, 'ObjType'])
            #print("from")
            #print(OSC_df.loc[idx, 'Source'])
            #print("distance between the two is:")
            #print(sep[idx])
            OSC_df.drop([idx], inplace=True)
            OSC_df.reset_index(inplace=True, drop=True)
        else:
            print("No matches, smallest separation is:")
            print(sep[idx].arcsec)

# re-write the dictionary such that
# no potential hosts exist that are not
# in our data frame of potential hosts
# (you can pass in an array, to_keep,
# of potential hosts to keep in the
# dictionary even if they're not in
# the dataframe)
def clean_dict(dic, df, to_keep):
    for name, host in dic.items():
            host = host.tolist()
            newHosts = host
            if len(host) > 0:
                for hostCandidate in np.array(host):
                    if hostCandidate not in np.array(df["objID"]) and hostCandidate not in np.array(to_keep):
                        newHosts.remove(hostCandidate)
                dic[name] = np.array(newHosts)
    return dic

# check to make sure that the list of all potential hosts
# in the dataframe matches all potential hosts in the
# dictionary - that the two describe the same hosts
def check_dict(dic, df):
    for name, host in dic.items():
            host = host.tolist()
            newHosts = host
            if len(host) > 0:
                for hostCandidate in np.array(host):
                    if hostCandidate not in np.array(df["objID"]):
                        print("Error: {}".format(hostCandidate))
                        print("objID not found!")

# clean the dictionary to match the datasetself
# note that if we're cutting by bestDetection, we find that
# we cut out many true hosts. So we only overwrite the
# list of potential hosts in this case if our new list isn't
# empty
def clean_dict(dic, df, to_keep=[],bestDetectionCut=0):
    for name, host in dic.items():
            host = host.tolist()
            newHosts = host
            if len(host) > 0:
                for hostCandidate in np.array(host):
                    if hostCandidate not in np.array(df["objID"]) and hostCandidate not in np.array(to_keep):
                        newHosts.remove(hostCandidate)
                if ((bestDetectionCut==0) | ((bestDetectionCut ==1) & (len(np.array(newHosts) != 0)))):
                    dic[name] = np.unique(newHosts)
                else:
                    dic[name] = np.unique(dic[name])
    return dic

def clean_df_from_dict(dic, df):
    allHosts = np.array([])
    for name, host in dic.items():
        if host.size>0:
            try:
                allHosts = np.concatenate((allHosts, np.array(host)),axis=0)
            except:
                continue
    allHosts = np.unique(allHosts)
    for index, row in df.iterrows():
        if (row['objID'] not in allHosts):
            df.drop(index, inplace=True)
    return df

#Because there are many duplicate entries in PS1 host table, follow this hierarchy:
# 1. If duplicate, remove non-primary detections
# 2. If still duplicate, remove NANs in yKronFlux, yskyErr, and yExtNSigma
# 3. If still duplicate, take the value with the smallest yKronFluxErr
def removePS1Duplicates(df):
    df.replace(-999.0,np.nan, inplace=True)
    new_df = []
    for hostCandidate in np.unique(df["objID"]):
        hostFrame = df[df["objID"] == hostCandidate]
        if len(hostFrame) > 1:
            newhostFrame = hostFrame[hostFrame["primaryDetection"] == 1]
            if len(newhostFrame) > 1:
                newhostFrame = newhostFrame[~np.isnan(newhostFrame["rApMag"])]
            if len(newhostFrame) > 1:
                newhostFrame = newhostFrame[newhostFrame["rApMagErr"] == np.min(newhostFrame["rApMagErr"])]
            if len(newhostFrame) > 0:
                new_df.append(newhostFrame)
            else:
                new_df.append(hostFrame)
        else:
            new_df.append(hostFrame)
    df = pd.concat(new_df)
    return df

# TODO add description
def getColors(df):
    df.replace(-999, np.nan, inplace=True)
    df.replace(999, np.nan, inplace=True)
    # create color attributes for all hosts
    df["i-z"] = df["iApMag"] - df["zApMag"]
    df["g-r"]= df["gApMag"] - df["rApMag"]
    df["r-i"]= df["rApMag"] - df["iApMag"]
    df["g-i"] = df["gApMag"] - df["iApMag"]
    df["z-y"] = df["zApMag"] - df["yApMag"]

    df['g-rErr'] = np.sqrt(df['gApMagErr']**2 + df['rApMagErr']**2)
    df['r-iErr'] = np.sqrt(df['rApMagErr']**2 + df['iApMagErr']**2)
    df['i-zErr'] = np.sqrt(df['iApMagErr']**2 + df['zApMagErr']**2)
    df['z-yErr'] = np.sqrt(df['zApMagErr']**2 + df['yApMagErr']**2)

    # To be sure we're getting physical colors
    df.loc[df['i-z'] > 100, 'i-z'] = np.nan
    df.loc[df['i-z'] < -100, 'i-z'] = np.nan
    df.loc[df['g-r'] > 100, 'i-z'] = np.nan
    df.loc[df['g-r'] < -100, 'i-z'] = np.nan

    # and PSF - Kron mag "morphology" information
    df["gApMag_gKronMag"] = df["gApMag"] - df["gKronMag"]
    df["rApMag_rKronMag"] = df["rApMag"] - df["rKronMag"]
    df["iApMag_iKronMag"] = df["iApMag"] - df["iKronMag"]
    df["zApMag_zKronMag"] = df["zApMag"] - df["zKronMag"]
    df["yApMag_yKronMag"] = df["yApMag"] - df["yKronMag"]
    #df["gApMag_rApMag"] = df["gApMag"] - df["rApMag"]
    #df["iApMag_zApMag"] = df["iApMag"] - df["zApMag"]

    # to be sure we're getting physical mags
    df.loc[df['iApMag_iKronMag'] > 100, 'iApMag_iKronMag'] = np.nan
    df.loc[df['iApMag_iKronMag'] < -100, 'iApMag_iKronMag'] = np.nan
    df.loc[df['iApMag'] > 100, 'iApMag'] = np.nan
    df.loc[df['iApMag'] < -100, 'iApMag'] = np.nan
    return df

# TODO Add description
def makeCuts(df,cuts=[],dict=""):
    for cut in cuts:
        if cut == "n":
            df = df[df['nDetections'] >= 1]
            df = df[df['ng'] >= 1]
            df = df[df['nr'] >= 1]
            df = df[df['ni'] >= 1]
            #df = df[df['nz'] >= 1]
            #df = df[df['ny'] >= 1]
        elif cut == "quality":
            df = df[df["qualityFlag"] < 128]
        elif cut == "coords":
            df = df.dropna(subset=['raMean', 'decMean'])
        elif cut == "mag":
            df = df[pd.notnull(df['gApMag'])]
            df = df[pd.notnull(df['rApMag'])]
            df = df[pd.notnull(df['iApMag'])]
            df = df[pd.notnull(df['zApMag'])]
            df = df[pd.notnull(df['yApMag'])]
        elif cut =="primary":
            if dict != "":
                df_mod = df[df["primaryDetection"] == 1]
                clean_dict(dict, df_mod, np.array([]),bestDetectionCut=1)
                clean_df_from_dict(dict, df)
            else:
                print("Can't make a primary detection cut - no dict provided!")
                return
        elif cut == "best":
            # subset by best detection, clean the dictionary, but include a flag that only
            # cleans the dict entry if there was a best detection in the field (else leave unchanged!)
            if dict != "":
                df_mod = df[df['bestDetection'] == 1]
                clean_dict(dict, df_mod, np.array([]),bestDetectionCut=1)
                clean_df_from_dict(dict, df)
            else:
                print("Can't make a best detection cut - no dict provided!")
                return
        elif cut=="duplicate":
            df = df.drop_duplicates(subset=['objID'])
        else:
            print("I didn't understand your cut!")
            return
    df.reset_index(drop=True, inplace=True)
    return df



import numpy as np
from astropy.table import Table
import requests
from PIL import Image
from io import BytesIO
import pylab
import os
import pandas as pd
import matplotlib.pyplot as plt
from astropy.io import fits
from astropy import units as u
from astropy.coordinates import SkyCoord
import re
from astro_ghost.PS1QueryFunctions import *
from astro_ghost.ghostHelperFunctions import *
import glob

# Get cooordinates associated with NGC 4568
supernovaCoord = SkyCoord(344.5011708333333*u.deg, 6.0634388888888875*u.deg, frame='icrs')
table = fullData()

galaxyCoord = SkyCoord(344.50184181*u.deg, 6.06983149*u.deg, frame='icrs')

getHostFromTransientCoords(supernovaCoord)
getHostFromTransientName("PTF10llv")

getHostStatsFromTransientName("SN2017hpi")
getHostStatsFromTransientCoords(supernovaCoord)

getTransientStatsFromHostName('UGC 12266')
getTransientStatsFromHostCoords(galaxyCoord)

getcolorim(ra, dec, size=tempSize, filters=band, format="png")
getHostImage('PTF10llv', save=0)

path = "/home/alexgagliano/Documents/Research/Transient_ML/tables/SNspectra/"
hostPath = '/home/alexgagliano/Documents/Research/Transient_ML_Box/hostSpectra/'

getTransientSpectra('SN2019ur', path)
getHostSpectra('2019dfi', hostPath)

coneSearchPairs(supernovaCoord, 1.e5)



from __future__ import print_function
import numpy as np
from astropy.table import Table
import requests
from PIL import Image
from io import BytesIO
import os
import pandas as pd
from astropy.io import fits
from astropy.visualization import PercentileInterval, AsinhStretch
from astropy.io import ascii
from astropy.table import Table
import sys
import re
import json
import requests
from datetime import datetime
from astropy import units as u
from astropy.coordinates import Angle
try: # Python 3.x
    from urllib.parse import quote as urlencode
    from urllib.request import urlretrieve
except ImportError:  # Python 2.x
    from urllib import pathname2url as urlencode
    from urllib import urlretrieve

try: # Python 3.x
    import http.client as httplib
except ImportError:  # Python 2.x
    import httplib
# std lib
from collections import OrderedDict
from os import listdir
from os.path import isfile, join
# 3rd party
from astropy import utils, io, convolution, wcs
from astropy.visualization import make_lupton_rgb
from astropy.coordinates import name_resolve
from pyvo.dal import sia
import pickle

def getAllPostageStamps(df, tempSize, path=".", verbose=0):
    for i in np.arange(len(df["raMean"])):
            tempRA = df.loc[i, 'raMean']
            tempDEC = df.loc[i, 'decMean']
            tempName = df.loc[i, 'TransientName']
            a = find_all(path+"/%s.png" % tempName, path)
            if not a:
                try:
                    img = getcolorim(tempRA, tempDEC, size=tempSize, filters="grizy", format="png")
                    img.save(path+"/%s.png" % tempName)
                    if verbose:
                        print("Saving postage stamp for the host of %s."% tempName)
                except:
                    continue

def preview_image(i, ra, dec, rad, band, save=1):
    a = find_all("PS1_ra={}_dec={}_{}arcsec_{}.fits".format(ra, dec, rad, band), ".")
    hdul = fits.open(a[0])
    image_file = get_pkg_data_filename(a[0])
    image_data = fits.getdata(image_file, ext=0)
    plt.figure()
    plt.imshow(image_data,cmap='viridis')
    plt.axis('off')
    #plt.colorbar()
    if save:
        plt.savefig("PS1_%i_%s.png" % (i, band))

def get_hosts(path, transient_fn, fn_Host, rad):
    transient_df = pd.read_csv(path+transient_fn)
    now = datetime.now()
    dateStr = "%i%.02i%.02i" % (now.year,now.month,now.day)
    dict_fn = fn_Host.replace(".csv", "") + ".p"
    find_host_info_PS1(transient_df, fn_Host, dict_fn, path, rad)
    host_df = pd.read_csv(path+fn_Host)
    host_df = host_df.drop_duplicates()
    host_df.to_csv(path+fn_Host[:-4]+"_cleaned.csv", index=False)
    return host_df

def find_all(name, path):
    result = []
    for root, dirs, files in os.walk(path):
        if name in files:
            result.append(os.path.join(root, name))
    return result

def getimages(ra,dec,size=240,filters="grizy", type='stack'):

    """Query ps1filenames.py service to get a list of images

    ra, dec = position in degrees
    size = image size in pixels (0.25 arcsec/pixel)
    filters = string with filters to include
    Returns a table with the results
    """

    service = "https://ps1images.stsci.edu/cgi-bin/ps1filenames.py"
    url = ("{service}?ra={ra}&dec={dec}&size={size}&format=fits"
           "&filters={filters}&type={type}").format(**locals())
    table = Table.read(url, format='ascii')
    return table


def geturl(ra, dec, size=240, output_size=None, filters="grizy", format="jpg", color=False, type='stack'):

    """Get URL for images in the table

    ra, dec = position in degrees
    size = extracted image size in pixels (0.25 arcsec/pixel)
    output_size = output (display) image size in pixels (default = size).
                  output_size has no effect for fits format images.
    filters = string with filters to include
    format = data format (options are "jpg", "png" or "fits")
    color = if True, creates a color image (only for jpg or png format).
            Default is return a list of URLs for single-filter grayscale images.
    Returns a string with the URL
    """

    if color and format == "fits":
        raise ValueError("color images are available only for jpg or png formats")
    if format not in ("jpg","png","fits"):
        raise ValueError("format must be one of jpg, png, fits")
    table = getimages(ra,dec,size=size,filters=filters, type=type)
    url = ("https://ps1images.stsci.edu/cgi-bin/fitscut.cgi?"
           "ra={ra}&dec={dec}&size={size}&format={format}").format(**locals())
    if output_size:
        url = url + "&output_size={}".format(output_size)
    # sort filters from red to blue
    flist = ["yzirg".find(x) for x in table['filter']]
    table = table[np.argsort(flist)]
    if color:
        if len(table) > 3:
            # pick 3 filters
            table = table[[0,len(table)//2,len(table)-1]]
        for i, param in enumerate(["red","green","blue"]):
            url = url + "&{}={}".format(param,table['filename'][i])
    else:
        urlbase = url + "&red="
        url = []
        for filename in table['filename']:
            url.append(urlbase+filename)
    return url


def getcolorim(ra, dec, size=240, output_size=None, filters="grizy", format="jpg"):

    """Get color image at a sky position

    ra, dec = position in degrees
    size = extracted image size in pixels (0.25 arcsec/pixel)
    output_size = output (display) image size in pixels (default = size).
                  output_size has no effect for fits format images.
    filters = string with filters to include
    format = data format (options are "jpg", "png")
    Returns the image
    """

    if format not in ("jpg","png"):
        raise ValueError("format must be jpg or png")
    url = geturl(ra,dec,size=size,filters=filters,output_size=output_size,format=format,color=True, type='stack')
    r = requests.get(url)
    im = Image.open(BytesIO(r.content))
    return im


def getgrayim(ra, dec, size=240, output_size=None, filter="g", format="jpg"):

    """Get grayscale image at a sky position

    ra, dec = position in degrees
    size = extracted image size in pixels (0.25 arcsec/pixel)
    output_size = output (display) image size in pixels (default = size).
                  output_size has no effect for fits format images.
    filter = string with filter to extract (one of grizy)
    format = data format (options are "jpg", "png")
    Returns the image
    """

    if format not in ("jpg","png"):
        raise ValueError("format must be jpg or png")
    if filter not in list("grizy"):
        raise ValueError("filter must be one of grizy")
    url = geturl(ra,dec,size=size,filters=filter,output_size=output_size,format=format)
    r = requests.get(url[0])
    im = Image.open(BytesIO(r.content))
    return im

def get_PS1_type(ra, dec, rad, band, type):
    fitsurl = geturl(ra, dec, size=rad, filters="{}".format(band), format="fits", type=type)
    fh = fits.open(fitsurl[0])
    fh.writeto('PS1_ra={}_dec={}_{}arcsec_{}_{}.fits'.format(ra, dec, int(rad*0.25), band, type))

def get_PS1_wt(ra, dec, rad, band):
    fitsurl = geturl(ra, dec, size=rad, filters="{}".format(band), format="fits", type='stack.wt')
    fh = fits.open(fitsurl[0])
    fh.writeto('PS1_ra={}_dec={}_{}arcsec_{}_wt.fits'.format(ra, dec, int(rad*0.25), band))

def get_PS1_mask(ra, dec, rad, band):
    fitsurl = geturl(ra, dec, size=rad, filters="{}".format(band), format="fits", type='stack.mask')
    fh = fits.open(fitsurl[0])
    fh.writeto('PS1_ra={}_dec={}_{}arcsec_{}_mask.fits'.format(ra, dec, int(rad*0.25), band))

def get_PS1_Pic(objID, ra, dec, rad, band, safe=False):
    fitsurl = geturl(ra, dec, size=rad, filters="{}".format(band), format="fits")
    fh = fits.open(fitsurl[0])
    if safe==True:
        fh.writeto('PS1_{}_{}arcsec_{}.fits'.format(objID, int(rad*0.25), band))
    else:
        fh.writeto('PS1_ra={}_dec={}_{}arcsec_{}.fits'.format(ra, dec, int(rad*0.25), band))

# Data Lab
#from dl import queryClient as qc
#from dl.helpers.utils import convert

# set up Simple Image Access (SIA) service
DEF_ACCESS_URL = "http://datalab.noao.edu/sia/des_dr1"
svc = sia.SIAService(DEF_ACCESS_URL)

##################### PS1 HELPER FUNCTIONS ############################################
def ps1metadata(table="mean",release="dr1",baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs"):
    """Return metadata for the specified catalog and table

    Parameters
    ----------
    table (string): mean, stack, or detection
    release (string): dr1 or dr2
    baseurl: base URL for the request

    Returns an astropy table with columns name, type, description
    """

    checklegal(table,release)
    url = "{baseurl}/{release}/{table}/metadata".format(**locals())
    r = requests.get(url)
    r.raise_for_status()
    v = r.json()
    # convert to astropy table
    tab = Table(rows=[(x['name'],x['type'],x['description']) for x in v],
               names=('name','type','description'))
    return tab


def mastQuery(request):
    """Perform a MAST query.

        Parameters
        ----------
        request (dictionary): The MAST request json object

        Returns head,content where head is the response HTTP headers, and content is the returned data"""

    server='mast.stsci.edu'

    # Grab Python Version
    version = ".".join(map(str, sys.version_info[:3]))

    # Create Http Header Variables
    headers = {"Content-type": "application/x-www-form-urlencoded",
               "Accept": "text/plain",
               "User-agent":"python-requests/"+version}

    # Encoding the request as a json string
    requestString = json.dumps(request)
    requestString = urlencode(requestString)

    # opening the https connection
    conn = httplib.HTTPSConnection(server)

    # Making the query
    conn.request("POST", "/api/v0/invoke", "request="+requestString, headers)

    # Getting the response
    resp = conn.getresponse()
    head = resp.getheaders()
    content = resp.read().decode('utf-8')

    # Close the https connection
    conn.close()

    return head,content

def resolve(name):
    """Get the RA and Dec for an object using the MAST name resolver

    Parameters
    ----------
    name (str): Name of object

    Returns RA, Dec tuple with position"""

    resolverRequest = {'service':'Mast.Name.Lookup',
                       'params':{'input':name,
                                 'format':'json'
                                },
                      }
    headers,resolvedObjectString = mastQuery(resolverRequest)
    resolvedObject = json.loads(resolvedObjectString)
    # The resolver returns a variety of information about the resolved object,
    # however for our purposes all we need are the RA and Dec
    try:
        objRa = resolvedObject['resolvedCoordinate'][0]['ra']
        objDec = resolvedObject['resolvedCoordinate'][0]['decl']
    except IndexError as e:
        raise ValueError("Unknown object '{}'".format(name))
    return (objRa, objDec)

def checklegal(table,release):
    """Checks if this combination of table and release is acceptable

    Raises a VelueError exception if there is problem
    """

    releaselist = ("dr1", "dr2")
    if release not in ("dr1","dr2"):
        raise ValueError("Bad value for release (must be one of {})".format(', '.join(releaselist)))
    if release=="dr1":
        tablelist = ("mean", "stack")
    else:
        tablelist = ("mean", "stack", "detection")
    if table not in tablelist:
        raise ValueError("Bad value for table (for {} must be one of {})".format(release, ", ".join(tablelist)))

def ps1search(table="mean",release="dr1",format="csv",columns=None,baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs", verbose=False,**kw):
    """Do a general search of the PS1 catalog (possibly without ra/dec/radius)

    Parameters
    ----------
    table (string): mean, stack, or detection
    release (string): dr1 or dr2
    format: csv, votable, json
    columns: list of column names to include (None means use defaults)
    baseurl: base URL for the request
    verbose: print info about request
    **kw: other parameters (e.g., 'nDetections.min':2).  Note this is required!
    """

    data = kw.copy()
    if not data:
        raise ValueError("You must specify some parameters for search")
    checklegal(table,release)
    if format not in ("csv","votable","json"):
        raise ValueError("Bad value for format")
    url = "{baseurl}/{release}/{table}.{format}".format(**locals())
    if columns:
        # check that column values are legal
        # create a dictionary to speed this up
        dcols = {}
        for col in ps1metadata(table,release)['name']:
            dcols[col.lower()] = 1
        badcols = []
        for col in columns:
            if col.lower().strip() not in dcols:
                badcols.append(col)
        if badcols:
            raise ValueError('Some columns not found in table: {}'.format(', '.join(badcols)))
        # two different ways to specify a list of column values in the API
        # data['columns'] = columns
        data['columns'] = '[{}]'.format(','.join(columns))

# either get or post works
#    r = requests.post(url, data=data)
    r = requests.get(url, params=data)

    if verbose:
        print(r.url)
    r.raise_for_status()
    if format == "json":
        return r.json()
    else:
        return r.text


def ps1cone(ra,dec,radius,table="stack",release="dr1",format="csv",columns=None,baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs", verbose=False,**kw):
    """Do a cone search of the PS1 catalog

    Parameters
    ----------
    ra (float): (degrees) J2000 Right Ascension
    dec (float): (degrees) J2000 Declination
    radius (float): (degrees) Search radius (<= 0.5 degrees)
    table (string): mean, stack, or detection
    release (string): dr1 or dr2
    format: csv, votable, json
    columns: list of column names to include (None means use defaults)
    baseurl: base URL for the request
    verbose: print info about request
    **kw: other parameters (e.g., 'nDetections.min':2)
    """

    data = kw.copy()
    data['ra'] = ra
    data['dec'] = dec
    data['radius'] = radius
    return ps1search(table=table,release=release,format=format,columns=columns,
                    baseurl=baseurl, verbose=verbose, **data)

#########################END PS1 HELPER FUNCTIONS##############################################

def create_df(tns_loc):
    """Combine all supernovae data into dataframe"""
    files = [f for f in listdir(tns_loc) if isfile(join(tns_loc, f))]
    arr = []
    for file in files:
        tempPD = pd.read_csv(tns_loc+file)
        arr.append(tempPD)
    df = pd.concat(arr)
    df = df.loc[df['RA'] != '00:00:00.000']
    df = df.drop_duplicates()
    df = df.replace({'Anon.': ''})
    df = df.replace({'2019-02-13.49': ''})
    df = df.apply(lambda x: x.str.strip() if x.dtype == "object" else x)
    return df
    #df.to_csv('SNe_TNS_061019.csv')

def query_ps1_noname(RA, DEC, rad):
    #print("Querying PS1 for nearest host...")
    return ps1cone(RA,DEC,rad/3600,table="stack",release="dr1",format="csv",columns=None,baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs", verbose=False)

def query_ps1_name(name, rad):
    #print("Querying PS1 with host name!")
    [ra, dec] = resolve(name)
    return ps1cone(ra,dec,rad/3600,table="stack",release="dr1",format="csv",columns=None,baseurl="https://catalogs.mast.stsci.edu/api/v0.1/panstarrs", verbose=False)

# Queries PS1 to find host info for each transient
# Input: df - a dataframe of all spectroscopically classified transients in TNS
#        fn - the output data frame of all PS1 potential hosts
#        dict_fn - the dictionary matching candidate hosts in PS1 and transients
# Output: N/A
def find_host_info_PS1(df, fn, dict_fn, path, rad):
    i = 0
    """Querying PS1 for all objects within rad arcsec of SNe"""
    #os.chdir()
    # SN_Host_PS1 - the dictionary to map SN IDs to nearby obj IDs in PS1
    # EDIT - We now know there are MANY SNe without IDs!! This is pretty problematic, so we're going to switch to
    # keeping a dictionary between names and objIDs
    SN_Host_PS1 = {}
    # PS1_queries - an array of relevant PS1 obj info
    PS1_queries = []
    for j, row in enumerate(df.itertuples(), 1):
            if ":" in str(row.RA):
                tempRA = Angle(row.RA, unit=u.hourangle)
            else:
                tempRA = Angle(row.RA, unit=u.deg)
            tempDEC = Angle(row.DEC, unit=u.deg)
            tempName = row.HostName
            a = ''
            if(row.HostName == ''): # and (int(row.DEC[0:3]) > -30)):
                a = query_ps1_noname(tempRA.degree,tempDEC.degree, rad)
            else:
            #    try:
            #        a = query_ps1_name(tempName, rad)
            #    except:
                a = query_ps1_noname(tempRA.degree,tempDEC.degree, rad)
            if a:
                a = ascii.read(a)
                a = a.to_pandas()
                PS1_queries.append(a)
                SN_Host_PS1[row.Name] = np.array(a['objID'])
            else:
                SN_Host_PS1[row.Name] = np.array([])

            # Print status messages every 10 lines
            if j%10 == 0:
                print("Processed {} of {} lines!".format(j, len(df.Name)))
                #print(SN_Host_PS1)

            # Print every query to a file Note: this was done in order
            # to prevent the code crashing after processing 99% of the data
            # frame and losing everything. This allows for duplicates though,
            # so they should be removed before the file is used again
            if (len(PS1_queries) > 0):
                PS1_hosts = pd.concat(PS1_queries)
                PS1_hosts = PS1_hosts.drop_duplicates()
                PS1_queries = []
                if i == 0:
                    PS1_hosts.to_csv(path+fn, header=True, index=False)
                    i = 1
                else:
                    PS1_hosts.to_csv(path+fn, mode='a+', header=False, index=False)
            else:
                print("No potential hosts found for this object...")
            # Save host info
            if not os.path.exists(path+ '/dictionaries/'):
            	os.makedirs(path+'/dictionaries/')
            with open(path+"/dictionaries/" + dict_fn, 'wb') as fp:
                pickle.dump(SN_Host_PS1, fp, protocol=pickle.HIGHEST_PROTOCOL)



import os
import sys
from astro_ghost.PS1QueryFunctions import getAllPostageStamps
from astro_ghost.TNSQueryFunctions import getTNSSpectra
from astro_ghost.NEDQueryFunctions import getNEDSpectra
#from astro_ghost.ghostHelperFunctions import getTransientHosts, getGHOST
from astro_ghost.ghostHelperFunctions import *
from astropy.coordinates import SkyCoord
from astropy import units as u
import pandas as pd
from datetime import datetime

#we want to include print statements so we know what the algorithm is doing
verbose = 1

#download the database from ghost.ncsa.illinois.edu
#note: real=False creates an empty database, which
#allows you to use the association methods without
#needing to download the full database first
getGHOST(real=True, verbose=verbose)

#create a list of the supernova names, their skycoords, and their classes (these three are from TNS)
snName = ['SN 2012dt', 'SN 1998bn', 'SN 1957B']

snCoord = [SkyCoord(14.162*u.deg, -9.90253*u.deg, frame='icrs'), \
           SkyCoord(187.32867*u.deg, -23.16367*u.deg, frame='icrs'), \
           SkyCoord(186.26125*u.deg, +12.899444*u.deg, frame='icrs')]

# run the association algorithm!
# this first checks the GHOST database for a SN by name, then by coordinates, and
# if we have no match then it manually associates them.
hosts = getTransientHosts(snName, snCoord, verbose=verbose, starcut='normal')

#create directories to store the host spectra, the transient spectra, and the postage stamps
hSpecPath = "./hostSpectra/"
tSpecPath = "./SNspectra/"
psPath = "./hostPostageStamps/"
paths = [hSpecPath, tSpecPath, psPath]
for tempPath in paths:
    if not os.path.exists(tempPath):
        os.makedirs(tempPath)

now = datetime.now()
dateStr = "%i%.02i%.02i" % (now.year,now.month,now.day)
rad = 30 #arcsec
fn_SN = 'transients_%s.csv' % dateStr
transients = pd.read_csv("./transients_%s/tables/%s"%(dateStr,fn_SN))

#get postage stamps and spectra
getAllPostageStamps(hosts, 120, psPath, verbose) #get postage stamps of hosts
getNEDSpectra(hosts, hSpecPath, verbose) #get spectra of hosts
getTNSSpectra(transients, tSpecPath, verbose) #get spectra of transients (if on TNS)

# Helper functions for querying the database
supernovaCoord = [SkyCoord(344.5011708333333*u.deg, 6.0634388888888875*u.deg, frame='icrs')]
galaxyCoord = [SkyCoord(344.50184181*u.deg, 6.06983149*u.deg, frame='icrs')]
snName = ["PTF10llv"]
table = fullData()

# 1. Get the entry corresponding to a specific transient by its name (or coordinates)
#    note: The coordinate/name is passed as a list, so multiple entries can be
#          queried simultaneously
#    This function returns the matches as a pandas dataframe (df) along with
#    a list of the sources not found (by name or coordinate)
df, notFound = getDBHostFromTransientCoords(supernovaCoord)
df, notFound = getDBHostFromTransientName(snName)

# 2. Print summary statistics about a particular host galaxy system or set of systems from a supernova
getHostStatsFromTransientName(snName)
getHostStatsFromTransientCoords(supernovaCoord)

# 3. Get stats about the supernovae associated with a host galaxy
galaxyName = ['UGC 12266']
getTransientStatsFromHostName(galaxyName)
getTransientStatsFromHostCoords(galaxyCoord)

# 4. get an image of the field by coordinates
tempSize = 400 #size in pixels
band = ['grizy']
getcolorim(galaxyCoord[0].ra.deg, galaxyCoord[0].dec.deg, size=tempSize, filters=band, format="png")

# 5. get an image of the host galaxy system associated with a supernova (by supernova name)
getHostImage(snName, save=0)

# 6. Find all supernova-host galaxy matches within a certain search radius (in arcseconds)
coneSearchPairs(supernovaCoord[0], 1.e3)



from matplotlib import colors
from scipy import ndimage
from astropy.wcs import WCS
import matplotlib.patches as mpatches
from matplotlib.pyplot import figure
from astro_ghost.sourceCleaning import clean_dict
from astropy.io import ascii
from astropy.table import Table
import requests
from astropy import units as u
from astropy.coordinates import SkyCoord
from astropy.coordinates import Angle
from collections import OrderedDict
from astropy.utils.data import get_pkg_data_filename
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

def find_nearest(array, value):
    array = np.asarray(array)
    idx = (np.abs(array - value)).argmin()
    return idx, array[idx]

#Taken from https://www.eso.org/~ohainaut/ccd/sn.html
#Error on Mag  ~    1/  (S/N)
# So calculating SNR for each band as 1/PSFMagErr
# Estimate the SNR of each band and choose the bands
# with the highest SNR for the rest of our measurements
# host_df - the dataframe of hosts we're considering
def choose_band_SNR(host_df):
    bands = ['g', 'r', 'i', 'z', 'y']
    try:
        gSNR = float(1/host_df["gPSFMagErr"])
        rSNR = float(1/host_df["rPSFMagErr"])
        iSNR = float(1/host_df["iPSFMagErr"])
        zSNR = float(1/host_df["zPSFMagErr"])
        ySNR = float(1/host_df["yPSFMagErr"])

        SNR = np.array([gSNR, rSNR, iSNR, zSNR, ySNR])
        i = np.argmax(SNR)
    except:
        #if we have issues getting the band with the highest SNR, just use 'r'-band
        i = 1
    return bands[i]
#Plot the DLR on each of these, as vectors in direction of SNe (SNe as star)
def calc_DLR(ra_SN, dec_SN, ra_host, dec_host, r_a, r_b, source, best_band):
    # EVERYTHING IS IN ARCSECONDS

    ## taken from "Understanding Type Ia Supernovae Through Their Host Galaxies..." by Gupta
    #https://repository.upenn.edu/cgi/viewcontent.cgi?referer=https://www.google.com/&httpsredir=1&article=1916&context=edissertations
    xr = np.abs(ra_SN.deg - float(ra_host))*3600
    yr = np.abs(dec_SN.deg - float(dec_host))*3600

    SNcoord = SkyCoord(ra_SN, dec_SN, frame='icrs')
    hostCoord = SkyCoord(ra_host*u.deg, dec_host*u.deg, frame='icrs')
    sep = SNcoord.separation(hostCoord)
    dist = sep.arcsecond
    badR = 10000000000.0 # if we don't have spatial information, get rid of it #this
    # is good in that it gets rid of lots of artifacts without radius information
    #dist = float(np.sqrt(xr**2 + yr**2))


    XX = best_band + 'momentXX'
    YY = best_band + 'momentYY'
    XY = best_band + 'momentXY'

    if (np.float(source[XX]) != np.float(source[XX])) | (np.float(source[XY]) != np.float(source[XY])) | \
        (np.float(source[YY]) != np.float(source[YY])):
        return dist, badR

    U = np.float(source[XY])
    Q = np.float(source[XX]) - np.float(source[YY])
    if Q == 0:
        return dist, badR
    phi = 0.5*np.arctan(U/Q)
    kappa = Q**2 + U**2
    a_over_b = (1 + kappa + 2*np.sqrt(kappa))/(1 - kappa)

    gam = np.arctan(yr/xr)
    theta = phi - gam

    DLR = r_a/np.sqrt(((a_over_b)*np.sin(theta))**2 + (np.cos(theta))**2)

    R = float(dist/DLR)

    if (R != R):
        return dist, badR

    return dist, R

def plot_ellipse(ax, px, s, ra, dec, color):
    i=0
    size = px  #PS cutout image size, 240*sidelength in arcmin
    x0, y0 = ((ra-s['raMean'])*4*3600*np.cos(s['decMean']/180*np.pi)+(size/2)), (s['decMean']-dec)*4*3600+(size/2)
    i=i+1

    y, x = np.mgrid[0:size, 0:size]# 4 pixel for 1 arcsec for PS1, here image size is set to be 20"*20", depend on your cutout image size
    #make fitted image
    n_radius=2
    theta1 = s['phi']#rot angle
    a1= s['r_a']
    b1= s['r_b']
    e1 = mpatches.Ellipse((x0, y0), 4*n_radius*a1, 4*n_radius*b1, theta1, lw=2, ls='--', edgecolor=color,
                          facecolor='none',  label='source 1')
    ax.add_patch(e1)

def plot_DLR_vectors(path, transient, transient_df, host_dict_candidates, host_dict_final, host_df, R_dict, ra_dict, df = "TNS", dual_axes=0, scale=1, postCut=0):
    hostList = host_dict_candidates[str(transient)]
    #os.chdir(path)
    if type(hostList) is np.ndarray:
        if len(hostList) > 1:
            chosen = host_dict_final[transient]
        else:
            chosen = hostList[0]
    else:
        chosen = hostList
        hostList = np.array(hostList)
    band = 'r'
    px = int(240*scale)
    row = transient_df[transient_df['Name'] == transient]

    tempRA = Angle(row.RA, unit=u.hourangle)
    tempDEC = Angle(row.DEC, unit=u.degree)
    transientRA = tempRA.degree[0]
    transientDEC = tempDEC.degree[0]

    try:
        truehostRA  = host_df.loc[host_df['objID'] == chosen, 'raMean'].values[0]
        truehostDEC = host_df.loc[host_df['objID'] == chosen, 'decMean'].values[0]
        searchRA = truehostRA
        searchDEC = truehostDEC
    except:
        searchRA = transientRA
        searchDEC = transientDEC

    a = find_all("PS1_ra={}_dec={}_{}arcsec_{}.fits".format(searchRA, searchDEC, int(px*0.25), band), '.')
    if not a:
        get_PS1_Pic(searchRA, searchDEC, px, band)
        a = find_all("PS1_ra={}_dec={}_{}arcsec_{}.fits".format(searchRA, searchDEC, int(px*0.25), band), '.')
    hdu = fits.open(a[0])[0]
    image_data = fits.open(a[0])[0].data
    wcs = WCS(hdu.header)
    figure(num=None, figsize=(8, 6), facecolor='w', edgecolor='k')
    ax = plt.subplot(projection=wcs)
    ax.annotate(r'%s' % np.array(row.Name)[0], xy=(transientRA-150, transientDEC+25), xytext=(transientRA-150, transientDEC+5), fontsize=20, color='r')
    if (dual_axes):
        overlay = ax.get_coords_overlay('fk5')
        overlay['ra'].set_axislabel('RA', fontsize=16)
        overlay['dec'].set_axislabel('DEC', fontsize=16)
    else:
        ax.set_xlabel("RA", fontsize=16)
        ax.set_ylabel("DEC", fontsize=16)
    # plotting the transient
    ax.scatter(transientRA, transientDEC, transform=ax.get_transform('fk5'), marker='*', lw=4, s=200,
           color="#f3a712",zorder=100)
    for host in hostList:
        hostDF = host_df[host_df['objID'] == host]

        band = choose_band_SNR(hostDF)
        XX = hostDF[band + 'momentXX'].values[0]
        YY = hostDF[band + 'momentYY'].values[0]
        XY = hostDF[band + 'momentXY'].values[0]
        U = np.float(XY)
        Q = np.float(XX) - np.float(YY)
        if (Q == 0):
            r_a = 1.e-5
        else:
            phi = 0.5*np.arctan(U/Q)
        kappa = Q**2 + U**2
        a_over_b = (1 + kappa + 2*np.sqrt(kappa))/(1 - kappa)

        r_a = ra_dict[host]
        r_b = r_a/(a_over_b)

        hostDF['r_a'] = r_a
        hostDF['r_b'] = r_b
        hostDF['phi'] = phi
        hostRA = host_df.loc[host_df['objID'] == host,'raMean'].values[0]
        hostDEC = host_df.loc[host_df['objID'] == host,'decMean'].values[0]

        hostDLR = R_dict[host]
        c = '#666dc9'
        c2 = 'red'
        if (host == chosen):
            c = c2 = '#d308d0'
        plot_ellipse(ax, px, hostDF, searchRA, searchDEC, c)
        #plot_ellipse(ax, px, hostDF, transientRA, transientDEC, c)

        # in arcseconds
        dx = float(hostRA - transientRA)*3600
        dy = float(hostDEC - transientDEC)*3600

        dist = np.sqrt(dx**2 + dy**2)
        if hostDLR == 10000000000.0:
            hostDLR = 0.0
        else:
            hostDLR = dist/hostDLR
        #in arcseconds
        scale_factor = hostDLR/dist

        DLR_RA = float(hostRA) - dx*scale_factor/3600
        DLR_DEC = float(hostDEC) - dy*scale_factor/3600

        pointRA = [hostRA, DLR_RA]
        pointDEC = [hostDEC, DLR_DEC]

        ax.plot(pointRA, pointDEC, transform=ax.get_transform('fk5'), lw=2, color= c)
    ax.imshow(image_data, norm=colors.PowerNorm(gamma = 0.5, vmin=1, vmax=1.e4), cmap='gray_r')
    try:
        z_host = float(host_df[host_df["objID"] == chosen]['NED_redshift'].values[0])
        if(z_host == z_host):
            print(" z_host={}\n".format(z_host))
            #ax.annotate(r"$z_{Host}$ = {%.2f}".format(z_host),xy=(transientRA, transientDEC), xytext=(transientRA, transientDEC), fontsize=300)
            ax.annotate(r'$z_{Host}$ = %.3f' % z_host, xy=(transientRA-280, transientDEC+40), xytext=(transientRA-280, transientDEC+40), fontsize=20, color='k')
            #ax.text(transientRA+0.003, transientDEC+0.003, "$z_{Host}$ = {%.2f}".format(z_host), transform=ax.get_transform('fk5'), fontsize=10)
    except:
        host_redshift = 0
    if postCut:
        plt.savefig("PS1_ra={}_dec={}_{}arcsec_{}_postCut.pdf".format(transientRA, transientDEC, int(px*0.25), band))
    else:
        plt.savefig("PS1_ra={}_dec={}_{}arcsec_{}_DLR.pdf".format(transientRA, transientDEC, int(px*0.25), band))

# if todo == "s", save the dictionary and the list of
# remaining sources
# if todo == "r", return them
def chooseByDLR(path, hosts, transients, fn, orig_dict, dict_mod, todo="s"):
    if todo=="s":
        if not os.path.exists(path+'/dictionaries'):
             os.makedirs(path+'/dictionaries')
        if not os.path.exists(path+'/tables'):
             os.makedirs(path+'/tables')
    hosts['dist/DLR'] = np.nan
    hosts['dist'] = np.nan
    noHosts = []

    clean_dict(orig_dict, hosts, [])
    f = open(path+fn, 'w')
    GDflag = 0
    GD_SN = []
    for name, host in orig_dict.items():
        if (type(host) is not np.int64 and type(host) is not float):
             if (len(host.shape) > 0) and (host.shape[0] > 0):
                R_dict = {}
                ra_dict = {}
                dist_dict = {}
                host = np.array(host)
                if len(host)>0:
                    for tempHost in host:
                        noGood = 0
                        theta = 0
                        host_df = hosts[hosts["objID"] == tempHost]
                        transient_df = transients[transients["Name"] == str(name)]
                        band = choose_band_SNR(host_df)
                        dist = 1.e10
                        R = 1.e10
                        if np.float64(host_df[band + 'KronRad']) > 0.0:
                            r_a = r_b = np.float64(host_df[band + 'KronRad'])
                        else:
                            noGood = 1
                        if noGood:
                            # If no good, we want to artificially inflate the distance to the SN so that
                            # we don't incorrectly pick this as our host
                            # (but the radius goes into plotting, so ra is artificially shrunk)
                            R = 1.e10
                            R_dict[tempHost] = R
                            noGood = 0
                            ra_dict[tempHost] = 0.05
                            GDflag = 1
                        else:
                            if ":" in str(transient_df["RA"].values):
                                ra_SN = Angle(transient_df["RA"].values, unit=u.hourangle)
                            else:
                                ra_SN = Angle(transient_df["RA"].values, unit=u.deg)
                            dec_SN = Angle(transient_df["DEC"].values, unit=u.degree)
                            ra_host = host_df['raMean']
                            dec_host = host_df['decMean']
                            if len(np.array(ra_SN)) > 1:
                                ra_SN = ra_SN[0]
                                dec_SN = dec_SN[0]
                            #print(tempHost)
                            #print(name)
                            dist, R = calc_DLR(ra_SN, dec_SN, ra_host, dec_host, r_a, r_b, host_df, band)
                            R_dict[tempHost] = R
                            ra_dict[tempHost] = r_a
                            dist_dict[tempHost] = dist
                        hosts.loc[hosts['objID'] == tempHost, 'dist/DLR'] = R
                        hosts.loc[hosts['objID'] == tempHost, 'dist'] = dist
                print(name, file=f)
                print("transient = \\", file=f)
                print(name, file=f)
                print("R_dict = \\", file=f)
                print(R_dict, file=f)
                print("ra_dict = \\", file=f)
                print(ra_dict, file=f)

                #subset so that we're less than 5 in DLR units
                chosenHost = min(R_dict, key=R_dict.get)
                if R_dict[chosenHost] > 5.0:
                    #If we can't find a host, say that this galaxy has no host
                    dict_mod[name] = np.nan
                    noHosts.append(name)
                    print("No host chosen! r/DLR > 5.0.", file=f)
                    continue
                else:
                    R_dict_sub = dict((k, v) for k, v in R_dict.items() if v <= 5.0)
                    #Sort from lowest to highest DLR value
                    R_dict_sub = {k: v for k, v in sorted(R_dict_sub.items(), key=lambda item: item[1])}
                    N = min(3, len(R_dict_sub.keys()))
                    R_dict_sub = dict(list(R_dict_sub.items())[:N])
                    if len(R_dict_sub.keys()) > 1:
                        gal_hosts = []
                        for key in R_dict_sub:
                            #print(hosts[hosts['objID'] == key]['NED_type'])
                            tempType = hosts[hosts['objID'] == key]['NED_type'].values[0]
                            if (tempType == "G"):
                                gal_hosts.append(key)
                        if len(gal_hosts) > 0:
                            chosenHost = gal_hosts[0]
                            print("Choosing the galaxy with the smallest DLR!", file=f)
                    dict_mod[name] = chosenHost
                    hosts.loc[hosts['objID'] == chosenHost, 'dist/DLR'] = R_dict[chosenHost]
                    hosts.loc[hosts['objID'] == chosenHost, 'dist'] = dist_dict[chosenHost]
                    if (GDflag):
                        GDflag = 0
                        print("Issue with DLR. Try Gradient Descent!", file=f)
                        GD_SN.append(name)
                    print(float(hosts[hosts['objID'] == chosenHost]['raMean']), float(hosts[hosts['objID'] == chosenHost]['decMean']), file=f)
                f.flush()
    f.close()
    if todo == "s":
        with open('../dictionaries/DLR_rankOrdered_hosts.p', 'wb') as fp:
            pickle.dump(dict_mod, fp, protocol=pickle.HIGHEST_PROTOCOL)
        hosts.to_csv("../tables/DLR_rankOrdered_hosts.csv")
        return
    elif todo =="r":
        return hosts, dict_mod, noHosts, GD_SN



import seaborn as sns
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib
import astropy.table as at
import scipy.interpolate as scinterp
import pkg_resources

# recalibration done via https://iopscience.iop.org/article/10.3847/0004-637X/822/2/66
# Hypercalibration: A Pan-starrs1-Based recalibration of the sloan digital sky survey
# photometry (Finkbeiner et al., 2016)
def convert_to_SDSS(conversions, g_iPS1, band, mPS1):
    a0 = conversions.loc[conversions['Band'] == band, 'a_0'].values[0]
    a1 = conversions.loc[conversions['Band'] == band, 'a_1'].values[0]
    a2 = conversions.loc[conversions['Band'] == band, 'a_2'].values[0]
    a3 = conversions.loc[conversions['Band'] == band, 'a_3'].values[0]
    mPS1_mSDSS = a0 + a1*g_iPS1 + a2*g_iPS1**2 + a3*g_iPS1**3
    mSDSS = mPS1 - (mPS1_mSDSS)
    return mSDSS

# Calculate 7DCD (actually a 4DCD here, because we only have these 4 colors) and
# store value in new column in dataframe
# stellar_loci - the table of stellar loci taken from PS1 (Tonry et al.)
# df - the dataframe of potential hosts
def calc_7DCD(df):
    #read the stellar locus table from SDSS
    df.replace(999.00, np.nan)
    df.replace(-999.00, np.nan)

    stream = pkg_resources.resource_stream(__name__, 'tonry_ps1_locus.txt')
    
    skt = at.Table.read(stream, format='ascii')

    gr = scinterp.interp1d(skt['ri'], skt['gr'], kind='cubic', fill_value='extrapolate')
    iz = scinterp.interp1d(skt['ri'], skt['iz'], kind='cubic', fill_value='extrapolate')
    zy = scinterp.interp1d(skt['ri'], skt['zy'], kind='cubic', fill_value='extrapolate')
    ri = np.arange(-0.4, 2.01, 0.001)

    gr_new = gr(ri)
    iz_new = iz(ri)
    zy_new = zy(ri)

    bands = ['g', 'r', 'i', 'z']

    #adding the errors in quadrature
    df["g-rErr"] =  np.sqrt(df["gApMagErr"].astype('float')**2 + df["rApMagErr"].astype('float')**2)
    df["r-iErr"] =  np.sqrt(df["rApMagErr"].astype('float')**2 + df["iApMagErr"].astype('float')**2)
    df["i-zErr"] =  np.sqrt(df["iApMagErr"].astype('float')**2 + df["zApMagErr"].astype('float')**2)
    df['z-yErr'] =  np.sqrt(df['zApMagErr'].astype('float')**2 + df['yApMagErr'].astype('float')**2)

    df["7DCD"] = np.nan
    df.reset_index(drop=True, inplace=True)
    for i in np.arange(len(df["i-z"])):
        temp_7DCD = []

        temp_7DCD_1val_gr = (df.loc[i,"g-r"] - gr_new)**2/df.loc[i, "g-rErr"]
        temp_7DCD_1val_ri = (df.loc[i,"r-i"] - ri)**2 /df.loc[i, "r-iErr"]
        temp_7DCD_1val_iz = (df.loc[i,"i-z"] - iz_new)**2/df.loc[i, "i-zErr"]
        temp_7DCD_1val_zy = (df.loc[i,"z-y"] - zy_new)**2/df.loc[i, "z-yErr"]

        temp_7DCD_1val = temp_7DCD_1val_gr + temp_7DCD_1val_ri + temp_7DCD_1val_iz + temp_7DCD_1val_zy

        df.loc[i,"7DCD"] = np.nanmin(np.array(temp_7DCD_1val))
    return df

def plotLocus(df, color=0, save=0, type="", timestamp=""):
    if color:
        plt.figure(figsize=(8,8))
        plt.scatter(df["i-z"], df["g-r"], c=df["7DCD"], s=2, alpha=0.8, norm=matplotlib.colors.LogNorm())
        plt.xlim(-0.75, 1)
        plt.clim(0.1, 1000)
        plt.ylim(-0.5, 2)
        plt.xlabel("i-z")
        plt.ylabel("g-r")
        cbar = plt.colorbar()
        cbar.set_label("3D Color Distance")
        if save:

            plt.savefig("PS1_%s_StellarLocus_%s_Colored.pdf"%(type, timestamp))
        else:
            plt.show()
    else:
        #read the stellar locus table from PS1
        skt = at.Table.read('./tonry_ps1_locus.txt', format='ascii')

        gr = scinterp.interp1d(skt['ri'], skt['gr'], kind='cubic', fill_value='extrapolate')
        iz = scinterp.interp1d(skt['ri'], skt['iz'], kind='cubic', fill_value='extrapolate')
        zy = scinterp.interp1d(skt['ri'], skt['zy'], kind='cubic', fill_value='extrapolate')
        ri = np.arange(-0.4, 2.01, 0.001)

        gr_new = gr(ri)
        iz_new = iz(ri)
        if type == 'Gals':
            c = '#087e8b'
        elif type == "Stars":
            c = '#ffbf00'
        else:
            c = 'violet'
        g = sns.JointGrid(x="i-z", y="g-r", data=df, height=9, ratio=5, xlim=(-0.4, 0.8), ylim=(-0.2, 2.0), space=0)
        g = g.plot_joint(sns.kdeplot, color=c,gridsize=200)
        plt.plot(iz_new, gr_new, '--', c='#8c837c', lw=2)
        plt.xlabel(r"$i-z$",fontsize=18)
        plt.ylabel(r"$g-r$",fontsize=18)
        g = g.plot_marginals(sns.kdeplot, color=c, shade=True)
        if save:
            g.savefig("PS1_%s_StellarLocus_%s.pdf"%(type, timestamp))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="astro_ghost",
    version="0.0.43",
    author="Alex Gagliano",
    author_email="gaglian2@illinois.edu",
    description="A package to associate transients with host galaxies, and a database of 16k SNe-host galaxies in PS1.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    package_data={
    '': ['Star_Galaxy_RealisticModel.sav','Star_Galaxy_IdealModel.sav', 'Star_Galaxy_RealisticModel_GHOST_PS1ClassLabels.sav','tonry_ps1_locus.txt'],
    },
    install_requires=['pandas', 'scikit-learn>=0.23.0', 'numpy', 'seaborn', 'matplotlib', 'joypy','astropy>=4.0', 'photutils', 'scipy', 'datetime', 'requests','imblearn','rfpimp','Pillow', 'pyvo', 'astroquery', 'certifi>=2020.06.20'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    include_package_data = True,
)
